const  WebExtensionsDivName = "BiscUsingWebExtensions";
var chrome = chrome || browser;

var DEBUG = false;

function debug(message, jsonobj){
	if (DEBUG){
		console.log("Content - " + message + (jsonobj ? JSON.stringify(jsonobj) : ""));
	}
}

//special case. we process in content script
function processGetExtensionVersion() {
	let version = chrome.runtime.getManifest().version;
	if (version) {
		let respFromCtxtScript = { error_code: 0, response: version};
		debug("response to version request: ", respFromCtxtScript);
		let detail = cloneInto({ direction: "from-content-script", message: respFromCtxtScript }, document.defaultView);
		let custEvent = new CustomEvent('content-script-response', { detail }); 
		document.dispatchEvent(custEvent);
    
		return;
	};
	
	console.log(`getExtManifest error: ${chrome.runtime.lastError.message}`);
	return;
}

document.addEventListener("page-script-request", event => {
	debug("received request from page");
	if (event.detail && event.detail.direction == "from-page-script") {
		let request = new Object();
		request.tracing = DEBUG;
		request.type = event.detail.message.calledMethodName;
		request.message = event.detail.message;
		
		//special case. no need to send to background
		if (request.message.method == "getExtensionVersion") {
			debug("request: ", request.message.method);
			processGetExtensionVersion();
			return;
		}

		debug("sending message to background script");
		chrome.runtime.sendMessage(request, response => {
			if (response) {
				debug("response from background script: ", response);
				let detail = cloneInto({ direction: "from-content-script", message: response }, document.defaultView);
				let custEvent = new CustomEvent('content-script-response', { detail });
				document.dispatchEvent(custEvent);
			} else {
				console.log(`${chrome.runtime.lastError.message}`);
			}
		});
	}
}, false);

document.addEventListener("toggle-tracing-request", event => {
	debug("received request from web page to toggle tracing");
	if (event.detail) {
		DEBUG = event.detail.tracing;
	}
}, false);

function isWebExtensionsEnabled() {
	let usingWebExtensionsDiv = document.getElementById(WebExtensionsDivName);
	return !!usingWebExtensionsDiv;
}

function enableWebExtensions() {

	if(isWebExtensionsEnabled()) { return; }

	//Create div elements
	let myWebExtensionDiv          = document.createElement("div");
	myWebExtensionDiv.id           = WebExtensionsDivName;
	document.body.appendChild(myWebExtensionDiv); 
}

function disableWebExtensions() {
	//Remove DIV element if it exist
	if(isWebExtensionsEnabled()) {
			let usingWebExtensionsDiv = document.getElementById(WebExtensionsDivName);
			usingWebExtensionsDiv.parentNode.removeChild(usingWebExtensionsDiv);     
	}
}

let observer = new MutationObserver(mutations => {
	for (let mutation of mutations) {
		if (mutation.target.nodeName === 'HTML') {
			let nodes = mutation.addedNodes;
			for (let i=0; i<nodes.length; i++) {
				if (nodes[i].nodeName === 'BODY' && document.body) {
					observer.disconnect();
					enableWebExtensions();
					break;
				}
			}
		}
	}
});
observer.observe(document, { childList: true, subtree: true });
